@@ -1,10 +1,4 @@
 import os
-import json
-import gzip
-import filesystem_query
-import experiment_years
-import esgf_query
-import copy
 
 def union_headers(diag_headers):
     #Create the diagnostic description dictionary:
@@ -29,19 +23,12 @@ def union_headers(diag_headers):
     for list_name in diag_desc.keys(): diag_desc[list_name]=list(set(diag_desc[list_name]))
     return diag_desc
 
-def find_optimset(diagnostic_headers_file, diagnostic_paths_file, gzip_flag):
+def find_optimset(diagnostic_headers_file, diagnostic_paths_file):
+    import valid_experiments
+    import json
     #Tree description following the CMIP5 DRS:
-    diag_tree_desc=['search',
-                    'center',
-                    'model',
-                    'experiment',
-                    'frequency',
-                    'realm',
-                    'mip',
-                    'rip',
-                    'version',
-                    'var'
-                    ]
+    diag_tree_desc=['search','center','model','experiment','frequency','realm','mip','rip',
+                    'version','var']
 
     #Load the diagnostic header file:
     paths_dict={}
@@ -72,25 +59,37 @@ def find_optimset(diagnostic_headers_file, diagnostic_paths_file, gzip_flag):
                                                                  paths_dict['diagnostic']['experiment_list'],
                                                                  diag_tree_desc[1:]
                                                                  )
-    #Define the tree strucuture in the output file
-    diag_tree_desc_final=[  
-                            'center',
-                            'model',
-                            'experiment',
-                            'frequency',
-                            'realm',
-                            'mip',
-                            'rip',
-                            'var',
-                            'year',
-                            'month',
-                            'version',
-                            'file_type',
-                            'search',
+
+    #Find the list of center / model with all experiments and variables requested:
+    diag_tree_desc_path=[
+                            'center','model','experiment','frequency','realm','mip',
+                            'rip','var','version','search','file_type'
+                         ]
+    paths_dict=valid_experiments.intersection_path(paths_dict,diag_tree_desc,diag_tree_desc_path)
+
+    #Write dictionary to file using JSON encoding.
+    outfile = open(diagnostic_paths_file,'w')
+    json.dump(paths_dict,outfile)
+
+    outfile.close()
+
+def find_optimset_months(diagnostic_headers_file, diagnostic_paths_file, gzip_flag):
+    import gzip
+    import json
+    import valid_experiments
+    paths_dict=json.load(diagnostic_headers_file)
+
+    diag_tree_desc_path=[
+                            'center','model','experiment','frequency','realm','mip',
+                            'rip','var','version','search','file_type'
                          ]
     #Find the list of center / model with all the months for all the years / experiments and variables requested:
-    paths_dict=experiment_years.intersection(paths_dict,diag_tree_desc,diag_tree_desc_final)
-    
+    diag_tree_desc_months=[
+                            'experiment','center','model','rip','frequency','realm','mip',
+                            'var','version','year','month','file_type','search'
+                         ]
+    paths_dict=valid_experiments.intersection_months(paths_dict,diag_tree_desc_path,diag_tree_desc_months)
+
     #Write dictionary to file using JSON encoding. If the gzip flag was used,
     #output directly to a compressed file:
     if gzip_flag:
@@ -100,17 +99,22 @@ def find_optimset(diagnostic_headers_file, diagnostic_paths_file, gzip_flag):
     json.dump(paths_dict,outfile)
 
     outfile.close()
+    
 
 def search_filesystem(diag_desc,diag_tree_desc,top_path):
+    import filesystem_query
     return filesystem_query.descend_tree(diag_desc,diag_tree_desc,top_path)
 
 def search_esgf(search_path,file_type_list,variable_list,experiment_list,diag_tree_desc):
+    import esgf_query
     return esgf_query.create_dict(search_path,file_type_list,variable_list,experiment_list,diag_tree_desc)
 
 
 def retrieval(retrieval_desc):
+    import json
     #Reads the file created by find_optimset and gives the path to file:
     if retrieval_desc.in_diagnostic_pointers_file[-3:]=='.gz':
+        import gzip
         infile=gzip.open(retrieval_desc.in_diagnostic_pointers_file,'r')
     else:
         infile=open(retrieval_desc.in_diagnostic_pointers_file,'r')
@@ -128,7 +132,7 @@ def tree_retrieval(retrieval_desc,diag_desc,paths_dict):
             #If the level name was specified in the retrieval, use this specification:
             tree_retrieval(retrieval_desc,diag_desc,paths_dict[str(getattr(retrieval_desc,level_name))])
         elif level_name=='version':
-            #the 'version' field is perculiar. Here, we use the most recent, or largest version number:
+            #the 'version' field is peculiar. Here, we use the most recent, or largest version number:
             version_list=[]
             for version in paths_dict.keys():
                 if version[0]!='_':
@@ -148,25 +152,26 @@ def tree_retrieval(retrieval_desc,diag_desc,paths_dict):
                     tree_retrieval(retrieval_desc,diag_desc,paths_dict[level])
                     break
     else:
-        print paths_dict
+        print paths_dict[0]
     return
 
 def simulations(simulations_desc):
+    import json
     #Reads the file created by find_optimset and gives the path to file:
     if simulations_desc.in_diagnostic_pointers_file[-3:]=='.gz':
+        import gzip
         infile=gzip.open(simulations_desc.in_diagnostic_pointers_file,'r')
     else:
         infile=open(simulations_desc.in_diagnostic_pointers_file,'r')
     paths_dict=json.load(infile)
 
-    print paths_dict['simulations_list']
+    for item in paths_dict['simulations_list']:
+        print item
     return
 
 
 def main():
-    import sys
     import argparse 
-    import shutil
     import textwrap
 
     #Option parser
@@ -192,7 +197,9 @@ def main():
     epilog_optimset=textwrap.dedent(epilog+'\n\nThe output can be pretty printed by using:\n\
                           cat out_paths_file | python -mjson.tool')
     optimset_parser=subparsers.add_parser('optimset',
-                                           help='Find the optimal set of models for a diagnostic',
+                                           help='Returns pointers to models that have all the requested experiments and variables.\n\
+                                                 It is good practice to check the results with \'simulations\' before\n\
+                                                 proceeding with \'optimset_months\'.',
                                            epilog=epilog_optimset,
                                            formatter_class=argparse.RawTextHelpFormatter
                                          )
@@ -201,7 +208,25 @@ def main():
                                  help='Diagnostic headers file (input)')
     optimset_parser.add_argument('out_paths_file',
                                  help='Diagnostic paths file (output)')
-    optimset_parser.add_argument('-z','--gzip',
+    #Find Optimset Months
+    epilog_optimset_months=textwrap.dedent(epilog+'\n\nThe output can be pretty printed by using:\n\
+                          cat out_paths_file | python -mjson.tool')
+    optimset_months_parser=subparsers.add_parser('optimset_months',
+                                           help='Take as an input the results from \'optimset\'.\n\
+                                                 Returns pointers to models that have all the\n\
+                                                 requested experiments and variables for all requested years.\n\
+                                                 It is required to use the \'retrieve\' command.\n\
+                                                 It can be slow, particularly if \'OPeNDAP\' files are\n\
+                                                 requested.',
+                                           epilog=epilog_optimset_months,
+                                           formatter_class=argparse.RawTextHelpFormatter
+                                         )
+    optimset_months_parser.add_argument('in_diagnostic_headers_file',
+                                 type=argparse.FileType('r'),
+                                 help='Diagnostic headers file (input)')
+    optimset_months_parser.add_argument('out_paths_file',
+                                 help='Diagnostic paths file (output)')
+    optimset_months_parser.add_argument('-z','--gzip',
                                  default=False, action='store_true',
                                  help='Compress the output using gzip. Because the output is mostly repeated text, this leads to large compression.')
 
@@ -229,9 +254,9 @@ def main():
                                  action='store_true',
                                  help='prints only the file_type if selected')
 
-    #Retrieve data
+    #Simulations
     simulations_parser=subparsers.add_parser('simulations',
-                                           help='Prints the (center,model,rip) triples available in the pointers file.'
+                                           help='Prints the (center_model_rip) triples available in the pointers file.'
                                            )
     simulations_parser.add_argument('in_diagnostic_pointers_file',
                                  help='Diagnostic headers file with data pointers (input)')
@@ -239,7 +264,9 @@ def main():
     options=parser.parse_args()
 
     if options.command=='optimset':
-        find_optimset(options.in_diagnostic_headers_file, options.out_paths_file, options.gzip)
+        find_optimset(options.in_diagnostic_headers_file, options.out_paths_file)
+    if options.command=='optimset_months':
+        find_optimset_months(options.in_diagnostic_headers_file, options.out_paths_file, options.gzip)
     elif options.command=='retrieve':
         retrieval(options)
     elif options.command=='simulations':
