@@ -25,6 +25,7 @@ def load_database(diag_tree_desc):
             *(sqlalchemy.Column(diag_tree, sqlalchemy.String(255)) for diag_tree in diag_tree_desc)
             )
     metadata.create_all()
+    sqlalchemy.orm.clear_mappers()
     sqlalchemy.orm.mapper(File_Expt,time_db)
 
     session = sqlalchemy.orm.create_session(bind=engine, autocommit=False, autoflush=True)
@@ -59,6 +60,7 @@ def experiment_variable_search(session,file_expt,search_path,file_type_list,expe
                         experiment=experiment,
                         time_frequency=frequency,
                         realm=realm,
+                        cmor_table=mip,
                         variable=var_name)
 
     keys_dict={}
@@ -66,7 +68,6 @@ def experiment_variable_search(session,file_expt,search_path,file_type_list,expe
     keys_dict['var']=var_name
     keys_dict['realm']=realm
     keys_dict['frequency']=frequency
-    #keys_dict['mip']=mip
     #Should put this constraint in the search but we put it explicitly...
 
     remote_file_types=['HTTPServer','GridFTP']
